@@ -162,7 +162,7 @@ class settings(metaclass=settingsMeta):
             if database not in (not_set, None):  # type: ignore
                 raise InvalidArgument(
                     "derandomize=True implies database=None, so passing "
-                    f"database={database!r} too is invalid."
+                    f"{database=} too is invalid."
                 )
             database = None
 
@@ -191,7 +191,7 @@ class settings(metaclass=settingsMeta):
         if not callable(_test):
             raise InvalidArgument(
                 "settings objects can be called as a decorator with @given, "
-                f"but decorated test={test!r} is not callable."
+                f"but decorated {test=} is not callable."
             )
         if inspect.isclass(test):
             from hypothesis.stateful import RuleBasedStateMachine
@@ -231,6 +231,7 @@ class settings(metaclass=settingsMeta):
         cls,
         name,
         description,
+        *,
         default,
         options=None,
         validator=None,
@@ -465,9 +466,10 @@ class HealthCheck(Enum, metaclass=HealthCheckMeta):
     def all(cls) -> List["HealthCheck"]:
         # Skipping of deprecated attributes is handled in HealthCheckMeta.__iter__
         note_deprecation(
-            f"`Healthcheck.all()` is deprecated; use `list(HealthCheck)` instead.",
+            "`Healthcheck.all()` is deprecated; use `list(HealthCheck)` instead.",
             since="2023-04-16",
             has_codemod=True,
+            stacklevel=1,
         )
         return list(HealthCheck)
 
@@ -601,6 +603,7 @@ def validate_health_check_suppressions(suppressions):
                 f"The {s.name} health check is deprecated, because this is always an error.",
                 since="2023-03-15",
                 has_codemod=False,
+                stacklevel=2,
             )
     return suppressions
 
@@ -687,7 +690,9 @@ The default is ``True`` if the ``CI`` or ``TF_BUILD`` env vars are set, ``False`
 settings.lock_further_definitions()
 
 
-def note_deprecation(message: str, *, since: str, has_codemod: bool) -> None:
+def note_deprecation(
+    message: str, *, since: str, has_codemod: bool, stacklevel: int = 0
+) -> None:
     if since != "RELEASEDAY":
         date = datetime.datetime.strptime(since, "%Y-%m-%d").date()
         assert datetime.date(2021, 1, 1) <= date
@@ -696,7 +701,7 @@ def note_deprecation(message: str, *, since: str, has_codemod: bool) -> None:
             "\n    The `hypothesis codemod` command-line tool can automatically "
             "refactor your code to fix this warning."
         )
-    warnings.warn(HypothesisDeprecationWarning(message), stacklevel=2)
+    warnings.warn(HypothesisDeprecationWarning(message), stacklevel=2 + stacklevel)
 
 
 settings.register_profile("default", settings())
