@@ -11,7 +11,6 @@
 import sys
 from collections import defaultdict
 from functools import lru_cache, reduce
-from itertools import groupby
 from os import sep
 from pathlib import Path
 
@@ -55,8 +54,12 @@ UNHELPFUL_LOCATIONS = (
     f"{sep}inspect.py",
     f"{sep}re.py",
     f"{sep}re{sep}__init__.py",  # refactored in Python 3.11
-    # Quite rarely, the first AFNP line is in Pytest's assertion-rewriting module.
+    f"{sep}warnings.py",
+    # Quite rarely, the first AFNP line is in Pytest's internals.
+    f"{sep}_pytest{sep}assertion{sep}__init__.py",
     f"{sep}_pytest{sep}assertion{sep}rewrite.py",
+    f"{sep}_pytest{sep}_io{sep}saferepr.py",
+    f"{sep}pluggy{sep}_result.py",
 )
 
 
@@ -118,10 +121,7 @@ EXPLANATION_STUB = (
 def make_report(explanations, cap_lines_at=5):
     report = defaultdict(list)
     for origin, locations in explanations.items():
-        report_lines = [
-            "        {}:{}".format(k, ", ".join(map(str, sorted(l for _, l in v))))
-            for k, v in groupby(locations, lambda kv: kv[0])
-        ]
+        report_lines = [f"        {fname}:{lineno}" for fname, lineno in locations]
         report_lines.sort(key=lambda line: (line.startswith(LIB_DIR), line))
         if len(report_lines) > cap_lines_at + 1:
             msg = "        (and {} more with settings.verbosity >= verbose)"
@@ -136,5 +136,5 @@ def explanatory_lines(traces, settings):
         return defaultdict(list)
     # Return human-readable report lines summarising the traces
     explanations = get_explaining_locations(traces)
-    max_lines = 5 if settings.verbosity <= Verbosity.normal else 100
+    max_lines = 5 if settings.verbosity <= Verbosity.normal else float("inf")
     return make_report(explanations, cap_lines_at=max_lines)
